#!/usr/bin/perl -w

#
# Template zur Dokumentation von Perlmodulen
#

#enter package name z.B. DSAP::Modulname, 
#package  

use strict;
use Getopt::Long;
use File::Basename;
use Net::LDAP;
use Net::LDAP::LDIF;
use Sys::Hostname;
use SmartClient::Config;
use SmartClient::Workstation;
use SmartClient::LdapUtil;
use Data::Dumper;


=head1 NAME

sc_getattr - get machine-specific attributes from LDAP

=begin comment

Skript zum Erfragen von Attributwerten, die einem Rechner zugeordnet sind

=end comment

=head1 SYNOPSIS

mkwsconfig

=begin comment

=end comment

=head1 DESCRIPTION

=begin comment 


=end comment

=head2 Allg. Beschreibung und Aufgaben


=head2 Funktionsweise / Design / Architektur / Sprache


=over 4

=item Funktionsweise


=item Architektur

Linux, i386

=begin comment

# Allgemein

=end comment



=head2 Sicherheitskonzept

=head2 Einordnung / Abhaengigkeiten / Voraussetzungen

=begin comment

Abhaenigkeiten von anderen Perlmodulen werden durch
require Statements sichergestellt.

=end comment

=head2 Interfaces(Programm/Benutzer)

=head2 Status (Fehler)

=cut


=head1 Methoden

=head2 Methode1

S<  >B<Methode> { I<parameter> =E<gt> I<value>, ... };

S<  >B<Methode> I<\@param1>, I<\@param2>[, I<$option1> ] [, I<$option2> ] ];  

=head1 Funktion

=head2 Funktion1


=head1 Globale Variablen

=head1 Konstanten

=cut

#Programmcode

sub usage()
{
	print "usage: sc_getattr.pl [--nodefault]\n";
        print "                     [--start user]\n";
	print "                     LDAP_ATTRIBUTE\n"; 
	print "                     [LDAP_ATTRIBUTE_OF_REFERENCED_LDAP_OBJECT]\n";
	print "\n";
	print "sc_getattr.pl is a wrapper for easier access to ldap_treesearch.\n";
	print "\n";
	print "Unless the option --start user is provided,\n";
	print "the start dn is set to the workstation dn.\n";
	print "With the option --start user, the script determine to user,\n"; 
	print "gets his DN from LDAP and starts there.\n";
        print "\n";
        print "If the options --nodefault is specified,\n";
        print "only the DN itself (workstation or user) is evaluated.\n";
	print "\n";
}



###############################################################################
#                                                                             #
# main program                                                                #
#                                                                             #
###############################################################################

my $config = SmartClient::Config->new();
if( ! defined $config ) {
	exit( "config file not found");
}

# get configuration
my $ldap_host = get_ldap_host();
my $ldap_base = $config->{'ALL_LDAPBASE'};

my $ldap_connection = Net::LDAP->new($ldap_host);
my $status = $ldap_connection->bind();

# command line options get stored in options hash
my %options = ();
my $result = GetOptions( \%options, 'nodefault', 'start=s' );

my $startdn;
if( $options{'start'} && ( "$options{'start'}" eq "user" ) ) {
	my $user = `id -un`;
	chomp $user;
	my $result = $ldap_connection->search( base => "$ldap_base",
						scope => 'sub',
						filter => "(uid=$user)" );
	if($result->code!=0) {
		`logger -t $0 "LDAP search error: $result->code \n"`;
	} else {
		#print Dumper( $result );
		my $entry = $result->entry(0);
		if( defined($entry) ) {
			$startdn = $entry->dn();
		}	
	}
} else {
	# get workstation DN
	$startdn = get_dn();
}

my $attrname;
my $indirect;
my $reference;
my $number_arguments = @ARGV;
if( $number_arguments == 1 ) {
	$attrname=$ARGV[0];
	$indirect=0;
} elsif( $number_arguments == 2 ) {
	$attrname=$ARGV[1];
	$reference=$ARGV[0];	
	$indirect=1;
} else {
	usage();
	exit 1;
}
my $entry;
my $nodefault = $options{'nodefault'};


my $searchfilter = $ARGV[0] . '=*';

if($nodefault)
{
    my $result = $ldap_connection->search( base => $startdn,
                            filter => $searchfilter);
    if($result->code!=0)
    {
            `logger -t $0 "LDAP search error: $result->code \n"`;
    }
    $entry=$result->entry(0);				
} else {			    
    $entry = ldap_treesearch( LDAP => $ldap_connection,
				STARTDN => $startdn,
				ROOTDN => $ldap_base,
				FILTER => $searchfilter);
}

if(!defined($entry))
{
	`logger -t $0  "LDAP search failed: $searchfilter\n"`;
	exit 1;
}

if($indirect)
{
	my $dn=$entry->get_value($reference);
	#print "getting object " . $dn . "\n";
	my $result = $ldap_connection->search( base => $dn,
						scope => 'base',
						filter => '(objectclass=*)' );
	if($result->code!=0)
	{
		`logger -t $0 "LDAP search error: $result->code \n"`;
	}
	$entry=$result->entry(0);
}

my $val;

if(defined($entry) && defined($val=$entry->get_value($attrname)))
{
	print $val . "\n"
}

$ldap_connection->unbind;
