#!/usr/bin/perl -w

#
# Template zur Dokumentation von Perlmodulen
#

#enter package name z.B. DSAP::Modulname,
package SmartClient::Pid;
use strict;
use warnings;

#use strict;

=head1 NAME

=begin comment

Grobkonzept

=end comment

=head1 SYNOPSIS

Anlegen einer PID-File unter /var/run/Prog.pid

=begin comment

  kurze Beschreibung, was das Programm macht, Feinkonzept

=end comment

=head1 DESCRIPTION

=begin comment 

  muss fuer das 

=end comment



=head2 Allg. Beschreibung und Aufgaben

=head2 Funktionsweise / Design / Architektur / Sprache

=over 4

=item Funktionsweise


=item Architektur

Linux, i386

=begin comment

# Allgemein

=end comment

=item Sprache

  Perl

=back



=head2 Sicherheitskonzept

=head2 Einordnung / Abhaengigkeiten / Voraussetzungen

=begin comment

Abhaenigkeiten von anderen Perlmodulen werden durch
require Statements sichergestellt.

=end comment

=head2 Interfaces(Programm/Benutzer)

=head2 Status (Fehler)

=head2 Version

=begin comment

  Versionsnummer fuer das Modul,  Feinkonzept,

=end comment

=cut

# version  of the package

our $VERSION = 0.1;

=head2 CVS Version

  $Id $


=head2 Installation und sonstige Hinweise

=begin comment

Feinkonzept

=end comment

=head1 AUTHOR

=begin comment

Name und e-mail

=end comment

SuSE Linux Solutions AG
mailto: rheinruhr@suse.de

=head1 SEE ALSO



=cut

# exportieren der Module
use Fcntl qw(:DEFAULT :flock);

# Festlegen der zu exportierenden Module, Methoden, Funktionen, Variablen, Konstanten

=head1 exportierte Module

S<  >B<Modul>

=cut

# Festlegen der optional zu exportierenden Module, Methoden, Funktionen, Variablen, Kostanten

#@EXPORT_OK;

=head1 optional exportierte Module

S<  >B<Modul>

S<  >B<Funktion>

S<  >B<Variable>

=cut

=head1 Modul

=begin comment

Beschreibung des Moduls

=end comment

S<  >B<Modul> { I<parameter> =E<gt> I<value>, ... };

S<  >B<Modul> I<\@param1>, I<\@param2>[, I<$option1> ] [, I<$option2> ] ];   

=cut

#Programmcode

##### Globals

my $DEFAULT_TIME      = 1;
my $DEFAULT_TIMEOUT   = 60;
my $DEFAULT_DIRECTORY = "/var/run/";

sub new
{
    my $package = shift;
    my $data    = {
        'TIME'      => undef,
        'TIMEOUT'   => undef,
        'ERROR'     => 0,
        'ERROR_MSG' => "",
        'FILE'      => undef,
        'DIRECTORY' => undef,
        'PROCNAME'  => undef,
        @_
    };
    $data->{'TIME'}      ||= $DEFAULT_TIME;
    $data->{'TIMEOUT'}   ||= $DEFAULT_TIMEOUT;
    $data->{'DIRECTORY'} ||= $DEFAULT_DIRECTORY;
    $data->{'FILE'}      ||= get_filename();
    $data->{'PROCNAME'}  ||= get_procname();
    create_file($data);
    return bless $data, $package;
}

=head1 Methoden

=head2 get_filename();

bildet den Dateiname der PID-Datei in der Form /var/run/programm.pid, wobei
Programm für $0 ohne Endung .pl steht.


=cut

sub get_filename
{
    my @a = split( /\//, $0 );

    my ( $str, @strs ) = split( /\./, $a[ @a - 1 ] );

    if ($str) {
        return $str;
    }
    return;
}

=head1 Methoden

=head2 get_procname();

bildet den Dateiname der PID-Datei in der Form /var/run/programm.pid, wobei
Programm für $0 ohne Endung .pl steht.


=cut

sub get_procname
{
    my @a = split( /\//, $0 );

    my $procname = pop(@a);

    if ($procname) {
        return $procname;
    }
    return;
}

=head2 create_file

Versucht eine PID-File zu erstellen setzt ERROR auf -1, wenn TIMOUT erreicht wird.

=cut

sub create_file
{
    my $self = shift;

    if ( !$self->{'FILE'} ) {
        $self->{'ERROR'}     = -2;
        $self->{'ERROR_MSG'} = "No Pid-File defined";
        return 0;
    }

    my $pidfile = $self->{'DIRECTORY'} . $self->{'FILE'} . ".pid";

    my $timebeg = time();
    while (1) {
        if ( sysopen( my $fh, $pidfile, O_RDWR | O_CREAT | O_EXCL ) ) {
            print $fh $$;
            close($fh);
            $self->{'ERROR'}     = 0;
            $self->{'ERROR_MSG'} = "";

            #print "created PID File ",$self->{'DIRECTORY'} . $self->{'FILE'}, "\n";
            return 1;
        } else {
            if ( sysopen( my $fe, $pidfile, O_RDONLY ) ) {

                #print "Can open File, $$\n";
                my $pid = <$fe>;
                if ( $pid and -f "/proc/$pid/stat" ) {

                    # syntax of /proc/*/stat has changed,
                    # so same program name isn't recognized.
                    # this check has been removed.
                    # a program running
                    # with the same PID as stored in the PID file
                    # results in an error
                    $self->{'ERROR'}     = -4;
                    $self->{'ERROR_MSG'} = "Already running with PID $pid";
                    return 1;
                } else {
                    ## programm is not running;
                    #print "Programm is not running, $$\n";
                    if ( unlink($pidfile) ) {
                        next;
                    } else {
                        $self->{'ERROR'} = -3;
                        $self->{'ERROR_MSG'} =
                          "Can\'t delete PID File: " . $pidfile;
                        return 0;
                    }
                }
                my $timenow = time();
                if ( ( $timenow - $timebeg ) > $self->{'TIMEOUT'} ) {
                    $self->{'ERROR'}     = -1;
                    $self->{'ERROR_MSG'} = "SmartClient::Pid: Global Timeout";

                    #print "Global Timeout\n";
                    return 0;
                }
            } else {
                $self->{'ERROR'}     = -5;
                $self->{'ERROR_MSG'} = "can't create or open PID file $pidfile";
                return 0;
            }
        }
        $self->{'ERROR'}     = -3;
        $self->{'ERROR_MSG'} = "SmartClient::Pid: Unknown Error";
        return 0;
    }
}

=head2 delete;

Löscht PID-File

=cut

sub delete
{
    my $self = shift;

    unlink( $self->{'DIRECTORY'} . $self->{'FILE'} . ".pid" );
}

=head2 getError

Get current error and error message

=cut

sub getError
{
    my ($self) = @_;
    return @{$self}{ 'ERROR', 'ERROR_MSG' };
}

=head1 Funktion

=head2 Funktion1


=head1 Globale Variablen


=head1 Konstanten


=head1 Tests

=head2 Test1

=over 4

=item Testbeschreibung 1


=item Testbericht


=back


=cut

#Programmcode
