#!/usr/bin/perl -w

package SmartClient::Util;
use strict;
use warnings;

=head1 NAME

SmartClient::Util

=head2 SVN Version

    $Id: Util.pm 7641 2011-12-07 13:10:45Z hmueller $

=begin comment

Grobkonzept

=end comment

B<SmartClient::Util>

=head1 SYNOPSIS

=begin comment

  kurze Beschreibung, was das Programm macht, Feinkonzept

=end comment

Hilfsfunktionen

=head1 DESCRIPTION

=begin comment 

  muss fuer das Grobkonzept ausgefuellt werden

=end comment

=head2 Allg. Beschreibung und Aufgaben

=head2 Funktionsweise / Design / Architektur / Sprache

=over 4

=item Funktionsweise

B<SmartClient::Util>  umfasst Funktionen, die von verschiedenen
Scripten verwendet werden.

=item Architektur

Linux, i386

=begin comment

# Allgemein

=end comment

=item Sprache

  Perl

=back



=head2 Sicherheitskonzept

=head2 Einordnung / Abhaengigkeiten / Voraussetzungen

=begin comment

Abhaenigkeiten von anderen Perlmodulen werden durch
require Statements sichergestellt.

=end comment

=head2 Interfaces(Programm/Benutzer)

=head2 Status (Fehler)

=head2 Version

=begin comment

  Versionsnummer fuer das Modul,  Feinkonzept,

=end comment

=cut

# version  of the package

our $VERSION = 0.1;

=head2 CVS Version

  $Id: Util.pm 7641 2011-12-07 13:10:45Z hmueller $

=head2 Installation und sonstige Hinweise

=begin comment

Feinkonzept

=end comment

=head1 AUTHOR

=begin comment

Name und e-mail

=end comment

Stephan Duehr

=head1 SEE ALSO

=cut

# exportieren der Module
use base 'Exporter';

use Data::Dumper;
use Log::Log4perl qw( :easy );
use Params::Validate qw(:all);
use File::stat;
use File::Path;
use IPC::Run;
use Time::HiRes qw( gettimeofday tv_interval usleep );
use Socket;
use SmartClient::IP;

# Festlegen der zu exportierenden Module, Methoden, Funktionen, Variablen, Konstanten
our @EXPORT =
  qw(check_parameter sleep_log run_command_timeout run_command retry convertCase compare stringArrayCompare isEqualArray getUndefinedHashKeys getFirstAvailable checkDir checkFileMod checkSymLink checkRemoveSymLink getUrlPort checkValidDnsName convertToValidDnsName getShortest wakeOnLan);

#our @EXPORT_OK;

=head1 optional exportierte Module

S<  >B<Modul>

S<  >B<Funktion>

S<  >B<Variable>

=cut

=head1 Modul

=begin comment

Beschreibung des Moduls

=end comment

keine Angabe

=cut

#Programmcode

=head1 Functions



=head2 check_parameter( @$ )

uses Params::Validate to check function parameter.
Normally, validate dies on errors.
To prevent this, validate is called by eval.
Error messages will be loggend und undef will be returned.
 
Only return href at the moment.

=over 4

=item parameter

    - array: parameter of a function
    - href:  description of required parameter (see Params::Validate)


=item return

    href of the parameter, potential filled with default values
    or undef on errors

=back

=cut

sub check_parameter
{
    my $logger = Log::Log4perl->get_logger();

    my $href_filter = pop;

    Params::Validate::validation_options( stack_skip => 3 );
    my %params = eval { my %p = validate( @_, $href_filter ) };

    if ( !%params ) {
        $logger->error($@);
    }

    return %params;

}

=head2 sleep_log

sleeps and logs what function has called the sleep

=over 4

=item parameter

    - float: sleep time in seconds (e.g. 0.1)
    - [String: additional log message]


=item return

    none

=back

=cut

sub sleep_log( $;$ )
{
    my $logger = Log::Log4perl->get_logger();

    my $duration = shift;
    my $add_log_msg = shift || "";

    #(my $package, my $filename, my $line, my $subroutine, my $hasargs, my $wantarray, my $evaltext, my $is_require, my $hints, my $bitmask) = caller( 1 );

    #$logger->debug( "sleep (${duration}s): $subroutine ($add_log_msg)" );
    usleep( $duration * 1000000.0 );
    return;
}

=head2 run_command_timeout

execute a non-interactive command 
and returns return code, stdout and stderr

Stops execution after (currently hard-coded) timeout


=over 4

=item parameter

command to execute,
with list of parameter

=item Return

  - integer: return code
  - string: stdout
  - string: stderr

=back

=cut

sub run_command_timeout
{
    my $logger = Log::Log4perl->get_logger();

    my $command = shift;
    my @args    = @_;

    # set timeout to 10 minutes
    my $timeout = 10 * 60;

    my ( $in, $stdout, $stderr );
    my $return_value;

    eval {
        my $process = IPC::Run::start(
            [ "bash", "-c", $command, @args ],
            \$in,
            \$stdout,
            \$stderr,
            (
                my $t = IPC::Run::timeout $timeout,
                name      => "stall",
                exception => "a timeout has occurred ($timeout seconds)"
            )
        );
        $process->finish;
        $return_value = $process->result(0);
        if ( $return_value == 0 ) {
            $logger->info( "successfully executed $command\n",
                "$stdout\n", "$stderr\n" );
        } else {
            $logger->error( "error during executing $command:\n",
                "$stdout\n", "$stderr\n" );
        }
    };    #eval
          # TODO: check error message on timeout
    if ($@) {
        $logger->error( "failed to execute "
              . join( " ", ( $command, @args ) ) . ": "
              . $@ );
    }

    return ( $return_value, $stdout, $stderr );
}

=head2 run_command

execute a non-interactive command and returns return code and output

=over 4

=item parameter

command to execute (single string including options)


=item return

return code,
list of output lines

=back

=cut

sub run_command
{
    my $command = shift;

    #print "executing command: " . $command . "\n";

    open( my $result_fh, '-|', "$command 2>&1" );
    my @result = <$result_fh>;
    close($result_fh);
    my $retcode = $? >> 8;

    #print @result;
    #if( $retcode ) { print "return code: " . $retcode . "\n"; }

    return ( $retcode, @result );
}

=head2 retry( % )

    tries to execute a function several times,
    until either it returns true
    or a timeout occurs

=over 4

=item parameter

        'function'
        'aref_args'
        'timeout'
        'delay'

=item return

    return value of function
    or 0 (false) on timeout

=back

=cut

sub retry( % )
{
    my $logger = Log::Log4perl->get_logger();

    my %params = (
        'function'  => undef,
        'aref_args' => [],
        'timeout'   => 60,
        'delay'     => 10,
        @_,
    );

    my $function = $params{'function'};
    my @args     = @{ $params{'aref_args'} };
    my $timeout  = $params{'timeout'};
    my $delay    = $params{'delay'};

    my $try = 1;

    # store start time
    my $t0 = [gettimeofday];

    do {
        my $result = &$function(@args);
        if ($result) {
            $logger->debug(
                "try $try has been successful for " . Dumper(@args) );
            return $result;
        }

        $logger->debug( "try $try not successful for "
              . Dumper(@args)
              . "Retrying in "
              . $delay
              . "s" );
        sleep($delay);
        $try++;

    } while ( tv_interval( $t0, [gettimeofday] ) < $timeout );

    $logger->warn( "failed for "
          . Dumper(@args)
          . " after "
          . tv_interval( $t0, [gettimeofday] )
          . "s" );
    return 0;
}

=head2 convertCase( $;$ )

execute a non-interactive command and returns return code and output

=over 4

=item parameter

    - scalar: value
    - boolean: casesensitive

=item return

if casesensitive is false, return value converted to lowercase

=back

=cut

sub convertCase( $;$ )
{
    my $value         = shift;
    my $casesensitive = shift;

    if ( $value && !$casesensitive ) {
        return lc($value);
    }

    return $value;
}

=head2 compare( $$;$ )

compares value1 and value2. Comparison is case insensitive, if third parameter is true

=over 4

=item parameter

    - scalar: value1
    - scalar: value2
    - boolean: case_ignore

=item return

    boolean

=back

=cut

sub compare( $$;$ )
{
    my $value1        = shift;
    my $value2        = shift;
    my $casesensitive = shift;

    if ( !( defined $value1 || defined $value2 ) ) {
        return 1;
    }

    if ( ( defined $value1 && defined $value2 ) !=
        ( defined $value1 || defined $value2 ) )
    {
        return;
    }

    return ( convertCase( $value1, $casesensitive ) eq
          convertCase( $value2, $casesensitive ) );
}

=head2 stringArrayCompare

Compare to arrays consisting of strings

=over 4

=item Parameter

2 references to arrays of strings

=item Return

1 (true) if arrays are identical, otherwise 0 (false)

=back

=cut

sub stringArrayCompare
{
    my ( $arr1_ref, $arr2_ref ) = @_;

    # if lenghts of arrays differ, they are not identical
    my $length1 = @$arr1_ref;
    my $length2 = @$arr2_ref;
    if ( $length1 != $length2 ) {
        return 0;
    }

    # arrays have same length, compare elements
    my $index = 0;
    foreach my $arr1_elem (@$arr1_ref) {
        if ( $arr1_elem ne $arr2_ref->[$index] ) {
            return 0;
        }
        $index++;
    }
    return 1;
}

=head2 isEqualArray

=over 4

=item Parameter

=item Return

=back

=cut

sub isEqualArray
{
    my $aref_one = shift;
    my $aref_two = shift;

    if ( (!defined $aref_one) || (!defined $aref_two) ) {
        if ( (!defined $aref_one) && (!defined $aref_two) ) {
            return 1;
        }
        return;
    }

    if ( ref $aref_one ne "ARRAY" or ref $aref_two ne "ARRAY" ) {
        return;
    }

    # check for same size
    if ( scalar @{$aref_one} != scalar @{$aref_two} ) {
        return;
    }

    my %one = map { lc($_) => 1 } @{$aref_one};
    my %two = map { lc($_) => 1 } @{$aref_two};

    # test each element from "one" if it is in "two"
    foreach my $o ( keys %one ) {
        if ( !exists $two{$o} ) {
            return;
        }
    }

    # seems they are equal
    return 1;
}

=head2 getUndefinedHashKeys( $$ )

    checks a attribute list for existence

=over 4

=item Parameter

    - hash reference: keys of hash will be checked
    - array reference: list of keys that should be checked

=item Return

    list of keys, that are not defined in hash

=back

=cut

sub getUndefinedHashKeys( $$ )
{
    my $href            = shift;
    my $aref_attributes = shift;

    my @result = ();

    foreach my $attribute ( @{$aref_attributes} ) {
        if ( not defined( $href->{$attribute} ) ) {
            push @result, $attribute;
        }
    }

    return @result;
}

=head2 getFirstAvailable

get first free key as number in a given hash reference

=head3 Example

    my $example_href = {'0' => 'zero', '1' => 'one', '3' => 'three'};
    my $first_free_key = getFirstAvailable('USED_NUMBERS_HREF' => $example_href); # should return 2

=over 4

=item parameter

    - USED_NUMBERS_HREF reference to a hash with numbers as keys
    - START first number to start the search
    - END last number to include in search

=item return

returns number

=back

=cut

sub getFirstAvailable
{
    my %param = (
        'USED_NUMBERS_HREF' => undef,
        'START'             => 0,
        'END'               => undef,
        @_,
    );

    my $used_numbers_href = $param{'USED_NUMBERS_HREF'};

    #sanity checks
    if ( ( ref($used_numbers_href) ne 'HASH' ) or ( !defined $param{'START'} ) )
    {
        return;
    }

    my $i = $param{'START'};
    while ( ( ( !defined $param{'END'} ) || $i <= $param{'END'} ) ) {
        if ( !$used_numbers_href->{$i} ) {

            # found a free number
            return $i;
        }
        $i++;
    }

    return;
}

=head2 checkDir( $ )

Creates a directory if it does not yet exist.

The File::Path function mkpath is used, which implements
creating the whole path also if underlying directories
do not yet exist (like mkdir -p). Because mkpath throws
a fatal error if anything goes wrong, eval is used for
preventing the program to die, but only if the directory
does not yet exist. See man File::Path for
details.

=over 4

=item Parameter

 - directory name

=item Return

1 (true) if the directory exists or could be created
0 (false) on error

=back

=cut

sub checkDir( $ )
{
    my $dir_path = shift;
    my $logger   = Log::Log4perl->get_logger();

    if ( -d "$dir_path" ) {
        return (1);
    }
    $logger->debug("checkDir creating $dir_path");
    eval { mkpath("$dir_path") };
    if ($@) {
        $logger->error("creating directory $dir_path failed: $@");
        return;
    }
    return 1;
}

=head2 checkFileMod( $$ )

    checks if a file was modified

=over 4

=item Parameter

    - filename
    - old modification time

=item Return

    boolean

=back

=cut

sub checkFileMod( $$ )
{
    my $filename      = shift;
    my $ref_mtime_old = shift;

    my $changed = undef;

    if ( stat($filename) ) {
        my $mtime_new = stat($filename)->mtime;
        $changed = ( $mtime_new > $$ref_mtime_old );
        if ($changed) {
            $$ref_mtime_old = $mtime_new;
        }

    }

    return $changed;
}

=head2 checkSymLink( $$ )

    creates a link between link source and link target.
    If target points already to file, 
    ir will be replaced

=over 4

=item Parameter

    - filename link source
    - filename link target

=item Return

    boolean

=back

=cut

sub checkSymLink( $$ )
{
    my $lnk_src = shift;
    my $lnk_tgt = shift;

    my $logger = Log::Log4perl->get_logger();

    # check if link target already exists
    if ( -l $lnk_tgt ) {

        # check if it points to the intended target already
        if ( readlink($lnk_tgt) eq $lnk_src ) {
            return 2;
        }

        # otherwise delete old link
        if ( !unlink($lnk_tgt) ) {
            $logger->warn("Error deleting Link $lnk_tgt");
            return 0;
        }
    }

    # link did not exist or was removed if it pointed to wrong target
    # create new link now
    my $symlink_rc = symlink( $lnk_src, $lnk_tgt );
    return $symlink_rc;
}

=head2 checkRemoveSymLink( $ )

Checks if the link exists and removes it if applicable.
Otherwise return true value quietly.

=over 4

=item Parameter

    - filename link target

=item Return

    boolean

=back

=cut

sub checkRemoveSymLink( $ )
{
    my $lnk_tgt = shift;

    my $logger = Log::Log4perl->get_logger();

    # check if link target already exists
    if ( -l $lnk_tgt ) {

        # otherwise delete old link
        if ( !unlink($lnk_tgt) ) {
            $logger->warn("Error deleting Link $lnk_tgt");
            return (0);
        }
    }

    # ok, link does not exist.
    return 1;
}

=head2 getUrlPort( $ )

Scans given C<URL> and for a portnumber. If no portnumber was found in URL, 
default portnumber 80 for C<http> (or 443 for C<https>) will be returned,
otherwise the found portnumber will be returned.

=over 4

=item Parameter

    - url

=item Return

    portnumber

=back

=cut

sub getUrlPort( $ )
{
    my $logger = Log::Log4perl->get_logger();

    # get URL where to listen.
    my $url = shift;
    my $port;

    if ( not( $url =~ m{https?://(.*):([0-9]*)} ) ) {
        if ( $url =~ m{http://(.*)} ) {
            $port = 80;
        } else {
            $port = 443;
        }
    } else {
        if ( $2 ne "" ) {
            $port = $2;
        }
   if ( !$port ) {
            $logger->warn("missing or malformed url $url");
        } else {
            $logger->info("using default port $port for url $url");
        }
   }
    return $port;
}

=head2 checkValidDnsName

checks if a given string is a valid dns name

=over 4

=item Parameter

string containing the dns name to check

=item Return

given parameter if it is a valid dns name, else undef

=back

=cut

sub checkValidDnsName
{
    my $dns_name = shift;

    if (   $dns_name
        && ( !ref $dns_name )
        && ( $dns_name =~ /^[A-Za-z0-9\.-]+$/o ) )
    {
        return $dns_name;
    }
    return;
}

=head2 convertToValidDnsName

convert a given string to a valid dns name

=head3 Example

  my $dns_name = convertToValudDnsName('test_123 with spaces');
  # should produce 'test-123-with-spaces'

=over 4

=item Parameter

string to convert

=item Return

converted dns name

=back

=cut

#TODO: umlaute will not work in function lc() when locale is not set apropriatly
sub convertToValidDnsName
{
    my $dns_name = shift;

    if ( $dns_name && ( !ref $dns_name ) ) {
        $dns_name = lc($dns_name);
        $dns_name =~ s/\s/-/go;
        $dns_name =~ s/_/-/go;
        $dns_name =~ s/ä/ae/go;
        $dns_name =~ s/ü/ue/go;
        $dns_name =~ s/ö/oe/go;
        $dns_name =~ s/ß/ss/go;
        if ( checkValidDnsName($dns_name) ) {
            return $dns_name;
        }
    }
    return;
}

=head2 getShortest( @ )

=over 4

=item parameter

    List

=item return

    shortest element from given list

=back

=cut

sub getShortest( @ )
{
    my @list = @_;

    my $shortest = shift @list;

    foreach my $i (@list) {
        if ( length($i) < length($shortest) ) {
            $shortest = $i;
        }
    }

    return $shortest;
}

=head2 wakeOnLan

    send a wake-on-lan packet

=over 4

=item parameter

    'ip_address'  IP-Address of Workstation
    'netmask'     Netmask of Workstation-Network 
    'mac_address' MAC-Address of Workstation
    'count'       Count of Pakets to send

=item return

    true if the WOL-Paket was successfully send, false otherwise

=back

=cut

sub wakeOnLan ( % )
{

    my $logger = Log::Log4perl->get_logger();

    my %params = (
        'ip_address'  => undef,
        'netmask'     => "255.255.255.0",
        'mac_address' => undef,
        'count'       => 1,
        @_,
    );

    if (    $params{'ip_address'}
        and $params{'netmask'}
        and $params{'mac_address'} =~ /^(?:[\da-f]{2}:){5}[\da-f]{2}$/i )
    {

        my $ipobj =
          SmartClient::IP->new( $params{'ip_address'}, $params{'netmask'} );
        if ( !ref $ipobj ) {
            $logger->warn( "creation of IP-Object with "
                  . $params{'ip_address'} . "/"
                  . $params{'netmask'}
                  . " failed" );
            return;
        }

        # Magic packet is 6 bytes of FF followed by the MAC address (without the :) 16 times
        ( my $mac_address = $params{'mac_address'} ) =~ tr/://d;
        my $magic = ( "\xff" x 6 ) . ( pack( 'H12', $mac_address ) x 16 );
        if ( !socket( S, PF_INET, SOCK_DGRAM, getprotobyname('udp') ) ) {
            $logger->warn("cannot create socket: $!");
            return;
        }
        setsockopt( S, SOL_SOCKET, SO_BROADCAST, pack( "L", 1 ) );
        my $send_ok = 1;
        for ( my $cnt = 1 ; $cnt <= $params{'count'} ; ++$cnt ) {
            for my $ip_address ( $ipobj->broadcast(), $params{'ip_address'} ) {
                if ( defined( my $inet_ip_addr = inet_aton($ip_address) ) ) {
                    $logger->debug(
                        "Sending wakeup packet to $ip_address at MAC address "
                          . $params{'mac_address'} );
                    if (
                        !defined(
                            send( S, $magic,
                                0, sockaddr_in( 0x2fff, $inet_ip_addr )
                            )
                        )
                      )
                    {
                        $logger->warn("error sending magic paket: $!");
                        $send_ok = 0;
                    }
                } else {
                    $logger->warn("Cannot resolve $ip_address");
                    $send_ok = 0;
                }
            }

            # a small pause
            usleep(100000);
        }
        close(S);
        return $send_ok;

    } else {
        $logger->warn(
            "parameter ip_address and/or mac_address is missing or malformed");
        return;
    }

}

1;
